#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Movable.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + VramRowSize * 1 + 32, "SCORE"); 
    PrintS(Vram + VramRowSize * 5 + 32, "HI-SCORE");
    PrintS(Vram + VramRowSize * 9 + 32, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 9 + 38, CurrentStage + 1);
    PrintS(Vram + VramRowSize * 12 + 32, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 21 + 33;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                byte c;
                c = Char_Remain;
                repeat (2) {
                    repeat (2) {
                        VPut(pVram, c);
                        ++pVram;
                        ++c;
                    }
                    pVram += VramRowSize - 2;
                }
                pVram += 3 - VramRowSize * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 2 + 34, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 6 + 34, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize * 12 + 37, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 10 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 12, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 11 + 13, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	L
        0x29, 0x20, 0x1d, 0x1d, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x21, 0x22, 0x22, 0x1e, 
        //	I
        0x21, 0x2a, 0x24, 0x1e, 
        0x1d, 0x29, 0x20, 0x1d, 
        0x1d, 0x29, 0x20, 0x1d, 
        0x21, 0x22, 0x22, 0x1e, 
        //	F
        0x29, 0x24, 0x22, 0x1e, 
        0x29, 0x28, 0x27, 0x1f, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x21, 0x1e, 0x1d, 0x1d, 
        //	T
        0x21, 0x2a, 0x24, 0x1e, 
        0x1d, 0x29, 0x20, 0x1d, 
        0x1d, 0x29, 0x20, 0x1d, 
        0x1d, 0x21, 0x1e, 0x1d, 
    };
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 8 + 8;
        p = TitleBytes;
        repeat (4) {
            repeat (4) {
                repeat (4) {
                    VPut(pVram, *p);
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 20 + 7, "PUSH TRIGGER BUTTON");
    PrintS(Vram + VramRowSize * 23 + 20, "INUFUTO 2022");
}
